@@ -179,8 +179,8 @@ class AsyncioWorkerProxy(WorkerProxy):
     async def _async_initialize(self):
         """Async initialization of the worker."""
         # Create worker wrapper with limits and retry logic if needed
-        # (limits and retry_config already processed by WorkerBuilder)
-        worker_cls = _create_worker_wrapper(self.worker_cls, self.limits, self.retry_config)
+        # (limits and retry_configs already processed by WorkerBuilder)
+        worker_cls = _create_worker_wrapper(self.worker_cls, self.limits, self.retry_configs)
 
         self._worker = worker_cls(*self.init_args, **self.init_kwargs)
 
@@ -344,7 +344,14 @@ class AsyncioWorkerProxy(WorkerProxy):
                             raise TypeError(f"fn must be callable, got {type(fn).__name__}")
 
                         # Apply retry logic if configured (for TaskWorker async functions)
-                        if self.retry_config is not None and self.retry_config.num_retries > 0:
+                        # Get retry config for "submit" method (fallback to "*")
+                        submit_retry_config = None
+                        if self.retry_configs is not None:
+                            submit_retry_config = self.retry_configs.get("submit") or self.retry_configs.get(
+                                "*"
+                            )
+
+                        if submit_retry_config is not None and submit_retry_config.num_retries > 0:
                             context = {
                                 "method_name": fn.__name__
                                 if hasattr(fn, "__name__")
@@ -352,7 +359,7 @@ class AsyncioWorkerProxy(WorkerProxy):
                                 "worker_class_name": "TaskWorker",
                             }
                             result = await execute_with_retry_async(
-                                fn, unwrapped_args, unwrapped_kwargs, self.retry_config, context
+                                fn, unwrapped_args, unwrapped_kwargs, submit_retry_config, context
                             )
                         else:
                             result = await fn(*unwrapped_args, **unwrapped_kwargs)
@@ -378,13 +385,18 @@ class AsyncioWorkerProxy(WorkerProxy):
                     raise TypeError(f"fn must be callable, got {type(fn).__name__}")
 
                 # Apply retry logic if configured (for TaskWorker sync functions)
-                if self.retry_config is not None and self.retry_config.num_retries > 0:
+                # Get retry config for "submit" method (fallback to "*")
+                submit_retry_config = None
+                if self.retry_configs is not None:
+                    submit_retry_config = self.retry_configs.get("submit") or self.retry_configs.get("*")
+
+                if submit_retry_config is not None and submit_retry_config.num_retries > 0:
                     context = {
                         "method_name": fn.__name__ if hasattr(fn, "__name__") else "anonymous_function",
                         "worker_class_name": "TaskWorker",
                     }
                     return execute_with_retry(
-                        fn, unwrapped_args, unwrapped_kwargs, self.retry_config, context
+                        fn, unwrapped_args, unwrapped_kwargs, submit_retry_config, context
                     )
                 else:
                     return fn(*unwrapped_args, **unwrapped_kwargs)
